@@ -131,6 +131,10 @@
         <source>Repeat new passphrase</source>
         <translation>Repita la nueva contraseña</translation>
     </message>
+    <message>
+        <source>Show passphrase</source>
+        <translation>Mostrar la frase de contraseña</translation>
+    </message>
     <message>
         <source>Encrypt wallet</source>
         <translation>Cifrar monedero</translation>
@@ -171,6 +175,14 @@
         <source>Wallet encrypted</source>
         <translation>Monedero cifrado</translation>
     </message>
+    <message>
+        <source>Wallet to be encrypted</source>
+        <translation>Billetera a ser cifrada</translation>
+    </message>
+    <message>
+        <source>Your wallet is now encrypted. </source>
+        <translation>Su billetera está ahora cifrada</translation>
+    </message>
     <message>
         <source>IMPORTANT: Any previous backups you have made of your wallet file should be replaced with the newly generated, encrypted wallet file. For security reasons, previous backups of the unencrypted wallet file will become useless as soon as you start using the new, encrypted wallet.</source>
         <translation>IMPORTANTE: Algunas copias de seguridad que hayas hecho de tu archivo de billetera deberían ser reemplazadas con la billetera encriptada generada recientemente. Por razones de seguridad, las copias de seguridad previas del archivo de billetera sin cifrar serán inútiles tan pronto uses la nueva billetera encriptada.</translation>
@@ -273,6 +285,18 @@
         <source>Open &amp;URI...</source>
         <translation>Abrir URI...</translation>
     </message>
+    <message>
+        <source>Create Wallet...</source>
+        <translation>Crear Billetera...</translation>
+    </message>
+    <message>
+        <source>Create a new wallet</source>
+        <translation>Crear una nueva billetera</translation>
+    </message>
+    <message>
+        <source>Network activity disabled.</source>
+        <translation>Actividad de red deshabilitada.</translation>
+    </message>
     <message>
         <source>Reindexing blocks on disk...</source>
         <translation>Reindexando bloques en disco...</translation>
@@ -385,6 +409,10 @@
         <source>Close wallet</source>
         <translation>Cerrar monedero</translation>
     </message>
+    <message>
+        <source>default wallet</source>
+        <translation>billetera por defecto</translation>
+    </message>
     <message>
         <source>No wallets available</source>
         <translation>Monederos no disponibles</translation>
@@ -695,6 +723,10 @@
 </context>
 <context>
     <name>OpenWalletActivity</name>
+    <message>
+        <source>default wallet</source>
+        <translation>billetera por defecto</translation>
+    </message>
     </context>
 <context>
     <name>OptionsDialog</name>
@@ -866,6 +898,9 @@
         <translation>Su balance actual total</translation>
     </message>
     </context>
+<context>
+    <name>PSBTOperationsDialog</name>
+    </context>
 <context>
     <name>PaymentServer</name>
     </context>
@@ -960,10 +995,6 @@
         <source>Block chain</source>
         <translation>Cadena de bloques</translation>
     </message>
-    <message>
-        <source>Current number of blocks</source>
-        <translation>Número actual de bloques</translation>
-    </message>
     <message>
         <source>Last block time</source>
         <translation>Hora del último bloque</translation>
@@ -1051,12 +1082,20 @@
         <source>Copy amount</source>
         <translation>Copiar cantidad</translation>
     </message>
-</context>
+    <message>
+        <source>Could not unlock wallet.</source>
+        <translation>No se pudo desbloquear la billetera.</translation>
+    </message>
+    </context>
 <context>
     <name>ReceiveRequestDialog</name>
     <message>
-        <source>QR Code</source>
-        <translation>Código QR</translation>
+        <source>Amount:</source>
+        <translation>Cuantía:</translation>
+    </message>
+    <message>
+        <source>Message:</source>
+        <translation>Mensaje:</translation>
     </message>
     <message>
         <source>Copy &amp;URI</source>
@@ -1070,23 +1109,7 @@
         <source>&amp;Save Image...</source>
         <translation>Guardar Imagen...</translation>
     </message>
-    <message>
-        <source>Address</source>
-        <translation>Dirección</translation>
-    </message>
-    <message>
-        <source>Amount</source>
-        <translation>Cantidad</translation>
-    </message>
-    <message>
-        <source>Label</source>
-        <translation>Etiqueta</translation>
-    </message>
-    <message>
-        <source>Wallet</source>
-        <translation>Monedero</translation>
-    </message>
-</context>
+    </context>
 <context>
     <name>RecentRequestsTableModel</name>
     <message>
@@ -1457,14 +1480,22 @@
     </context>
 <context>
     <name>WalletFrame</name>
-    </context>
+    <message>
+        <source>Create a new wallet</source>
+        <translation>Crear una nueva billetera</translation>
+    </message>
+</context>
 <context>
     <name>WalletModel</name>
     <message>
         <source>Send Coins</source>
         <translation>Enviar monedas</translation>
     </message>
-    </context>
+    <message>
+        <source>default wallet</source>
+        <translation>billetera por defecto</translation>
+    </message>
+</context>
 <context>
     <name>WalletView</name>
     <message>
@@ -1475,6 +1506,14 @@
         <source>Export the data in the current tab to a file</source>
         <translation>Exportar a un archivo los datos de esta pestaña</translation>
     </message>
+    <message>
+        <source>Error</source>
+        <translation>Error</translation>
+    </message>
+    <message>
+        <source>Backup Wallet</source>
+        <translation>Billetera de Respaldo</translation>
+    </message>
     <message>
         <source>Backup Failed</source>
         <translation>Copia de seguridad fallida</translation>
@@ -1491,7 +1530,11 @@
         <source>The wallet data was successfully saved to %1.</source>
         <translation>Los datos de la billetera fueron guardados exitosamente al %1</translation>
     </message>
-    </context>
+    <message>
+        <source>Cancel</source>
+        <translation>Cancelar</translation>
+    </message>
+</context>
 <context>
     <name>bitcoin-core</name>
     <message>
